@@ -21,24 +21,26 @@ from etree13 import ElementTree as ET
 
 
 ENV_WARNINGS = """\
-(%(root)s/autodoc_fodder.py:docstring of autodoc_fodder\\.MarkupError:2: \
-WARNING: Explicit markup ends without a blank line; unexpected \
-unindent\\.\\n?
-)?%(root)s/images.txt:9: WARNING: image file not readable: foo.png
-%(root)s/images.txt:23: WARNING: nonlocal image URI found: \
-http://www.python.org/logo.png
-%(root)s/includes.txt:\\d*: WARNING: Encoding 'utf-8-sig' used for \
-reading included file u'.*?wrongenc.inc' seems to be wrong, try giving an \
-:encoding: option\\n?
-%(root)s/includes.txt:4: WARNING: download file not readable: .*?nonexisting.png
-(%(root)s/markup.txt:357: WARNING: invalid single index entry u'')?
-(%(root)s/undecodable.txt:3: WARNING: undecodable source characters, replacing \
+(%(root)s/autodoc_fodder.py:docstring of autodoc_fodder.MarkupError:\\d+: \
+WARNING: duplicate object description of autodoc_fodder.MarkupError, other \
+instance in %(root)s/autodoc.rst, use :noindex: for one of them
+)?%(root)s/autodoc_fodder.py:docstring of autodoc_fodder.MarkupError:\\d+: \
+WARNING: Explicit markup ends without a blank line; unexpected unindent.
+%(root)s/index.rst:\\d+: WARNING: Encoding 'utf-8-sig' used for reading included \
+file u'%(root)s/wrongenc.inc' seems to be wrong, try giving an :encoding: option
+%(root)s/index.rst:\\d+: WARNING: image file not readable: foo.png
+%(root)s/index.rst:\\d+: WARNING: nonlocal image URI found: http://www.python.org/logo.png
+%(root)s/index.rst:\\d+: WARNING: download file not readable: %(root)s/nonexisting.png
+%(root)s/index.rst:\\d+: WARNING: invalid single index entry u''
+%(root)s/undecodable.rst:\\d+: WARNING: undecodable source characters, replacing \
 with "\\?": b?'here: >>>(\\\\|/)xbb<<<'
-)?"""
+"""
 
 HTML_WARNINGS = ENV_WARNINGS + """\
-%(root)s/images.txt:20: WARNING: no matching candidate for image URI u'foo.\\*'
-%(root)s/footnote.txt:60: WARNING: citation not found: missing
+%(root)s/index.rst:\\d+: WARNING: no matching candidate for image URI u'foo.\\*'
+%(root)s/index.rst:\\d+: WARNING: Could not lex literal_block as "c". Highlighting skipped.
+%(root)s/index.rst:\\d+: WARNING: unknown option: &option
+%(root)s/index.rst:\\d+: WARNING: citation not found: missing
 """
 
 if PY3:
@@ -79,8 +81,8 @@ HTML_XPATH = {
         (".//pre", u'Max Strauß'),
         (".//a[@href='_downloads/img.png']", ''),
         (".//a[@href='_downloads/img1.png']", ''),
-        (".//pre", u'"quotes"'),
-        (".//pre", u"'included'"),
+        (".//pre/span", u'"quotes"'),
+        (".//pre/span", u"'included'"),
         (".//pre/span[@class='s2']", u'üöä'),
         (".//div[@class='inc-pyobj1 highlight-text']//pre",
             r'^class Foo:\n    pass\n\s*$'),
@@ -149,8 +151,10 @@ HTML_XPATH = {
             "[@class='reference internal']/code/span[@class='pre']", '^with$'),
         (".//a[@href='#grammar-token-try_stmt']"
             "[@class='reference internal']/code/span", '^statement$'),
+        (".//a[@href='#some-label'][@class='reference internal']/span", '^here$'),
+        (".//a[@href='#some-label'][@class='reference internal']/span", '^there$'),
         (".//a[@href='subdir/includes.html']"
-            "[@class='reference internal']/em", 'Including in subdir'),
+            "[@class='reference internal']/span", 'Including in subdir'),
         (".//a[@href='objects.html#cmdoption-python-c']"
             "[@class='reference internal']/code/span[@class='pre']", '-c'),
         # abbreviations
@@ -207,7 +211,7 @@ HTML_XPATH = {
         (".//a[@href='#c.SphinxType']", ''),
         (".//a[@href='#c.sphinx_global']", ''),
         # test global TOC created by toctree()
-        (".//ul[@class='current']/li[@class='toctree-l1 current']/a[@href='']",
+        (".//ul[@class='current']/li[@class='toctree-l1 current']/a[@href='#']",
             'Testing object descriptions'),
         (".//li[@class='toctree-l1']/a[@href='markup.html']",
             'Testing various markup'),
@@ -272,6 +276,9 @@ HTML_XPATH = {
          'http://sphinx-doc.org/'),
         (".//a[@class='reference external'][@href='http://sphinx-doc.org/latest/']",
          'Latest reference'),
+        # Indirect hyperlink targets across files
+        (".//a[@href='markup.html#some-label'][@class='reference internal']/span",
+         '^indirect hyperref$'),
     ],
     'bom.html': [
         (".//title", " File with UTF-8 BOM"),
@@ -384,11 +391,10 @@ def check_extra_entries(outdir):
     assert (outdir / 'robots.txt').isfile()
 
 
-@gen_with_app(buildername='html', freshenv=True,  # use freshenv to check warnings
-              confoverrides={'html_context.hckey_co': 'hcval_co'},
-              tags=['testtag'])
-def test_html_output(app, status, warning):
+@with_app(buildername='html', testroot='warnings', freshenv=True)
+def test_html_warnings(app, status, warning):
     app.builder.build_all()
+
     html_warnings = warning.getvalue().replace(os.sep, '/')
     html_warnings_exp = HTML_WARNINGS % {
         'root': re.escape(app.srcdir.replace(os.sep, '/'))}
@@ -397,6 +403,11 @@ def test_html_output(app, status, warning):
         '--- Expected (regex):\n' + html_warnings_exp + \
         '--- Got:\n' + html_warnings
 
+
+@gen_with_app(buildername='html', tags=['testtag'],
+              confoverrides={'html_context.hckey_co': 'hcval_co'})
+def test_html_output(app, status, warning):
+    app.builder.build_all()
     for fname, paths in iteritems(HTML_XPATH):
         parser = NslessParser()
         parser.entity.update(html_entities.entitydefs)
@@ -555,7 +566,7 @@ def test_numfig_disabled(app, status, warning):
             yield check_xpath, etree, fname, xpath, check, be_found
 
 
-@gen_with_app(buildername='html', testroot='numfig',
+@gen_with_app(buildername='html', testroot='numfig', freshenv=True,
               confoverrides={'numfig': True})
 def test_numfig_without_numbered_toctree(app, status, warning):
     # remove :numbered: option
@@ -589,7 +600,7 @@ def test_numfig_without_numbered_toctree(app, status, warning):
             (".//li/a/span", '^Table:6$', True),
             (".//li/a/span", '^Listing 9$', True),
             (".//li/a/span", '^Code-6$', True),
-            ],
+        ],
         'foo.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Fig. 1 $', True),
@@ -615,7 +626,7 @@ def test_numfig_without_numbered_toctree(app, status, warning):
              "span[@class='caption-number']", '^Listing 3 $', True),
             (".//div[@class='code-block-caption']/"
              "span[@class='caption-number']", '^Listing 4 $', True),
-            ],
+        ],
         'bar.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Fig. 5 $', True),
@@ -686,7 +697,7 @@ def test_numfig_with_numbered_toctree(app, status, warning):
             (".//li/a/span", '^Table:2.2$', True),
             (".//li/a/span", '^Listing 1$', True),
             (".//li/a/span", '^Code-2.2$', True),
-            ],
+        ],
         'foo.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Fig. 1.1 $', True),
@@ -712,7 +723,7 @@ def test_numfig_with_numbered_toctree(app, status, warning):
              "span[@class='caption-number']", '^Listing 1.3 $', True),
             (".//div[@class='code-block-caption']/"
              "span[@class='caption-number']", '^Listing 1.4 $', True),
-            ],
+        ],
         'bar.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Fig. 2.1 $', True),
@@ -786,7 +797,7 @@ def test_numfig_with_prefix(app, status, warning):
             (".//li/a/span", '^Table:2.2$', True),
             (".//li/a/span", '^Code-1$', True),
             (".//li/a/span", '^Code-2.2$', True),
-            ],
+        ],
         'foo.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Figure:1.1 $', True),
@@ -812,7 +823,7 @@ def test_numfig_with_prefix(app, status, warning):
              "span[@class='caption-number']", '^Code-1.3 $', True),
             (".//div[@class='code-block-caption']/"
              "span[@class='caption-number']", '^Code-1.4 $', True),
-            ],
+        ],
         'bar.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Figure:2.1 $', True),
@@ -883,7 +894,7 @@ def test_numfig_with_secnum_depth(app, status, warning):
             (".//li/a/span", '^Table:2.1.2$', True),
             (".//li/a/span", '^Listing 1$', True),
             (".//li/a/span", '^Code-2.1.2$', True),
-            ],
+        ],
         'foo.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Fig. 1.1 $', True),
@@ -909,7 +920,7 @@ def test_numfig_with_secnum_depth(app, status, warning):
              "span[@class='caption-number']", '^Listing 1.1.2 $', True),
             (".//div[@class='code-block-caption']/"
              "span[@class='caption-number']", '^Listing 1.2.1 $', True),
-            ],
+        ],
         'bar.html': [
             (".//div[@class='figure']/p[@class='caption']/"
              "span[@class='caption-number']", '^Fig. 2.1.1 $', True),
@@ -950,16 +961,132 @@ def test_numfig_with_secnum_depth(app, status, warning):
             yield check_xpath, etree, fname, xpath, check, be_found
 
 
-@with_app(buildername='html')
-def test_jsmath(app, status, warning):
+@gen_with_app(buildername='singlehtml', testroot='numfig',
+              confoverrides={'numfig': True})
+def test_numfig_with_singlehtml(app, status, warning):
     app.builder.build_all()
-    content = (app.outdir / 'math.html').text()
 
-    assert '<div class="math">\na^2 + b^2 = c^2</div>' in content
-    assert '<div class="math">\n\\begin{split}a + 1 &lt; b\\end{split}</div>' in content
-    assert ('<span class="eqno">(1)</span><div class="math" id="equation-foo">\n'
-            'e^{i\\pi} = 1</div>' in content)
-    assert ('<span class="eqno">(2)</span><div class="math">\n'
-            'e^{ix} = \\cos x + i\\sin x</div>' in content)
-    assert '<div class="math">\nn \\in \\mathbb N</div>' in content
-    assert '<div class="math">\na + 1 &lt; b</div>' in content
+    expects = {
+        'index.html': [
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 1 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 2 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 1 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 2 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 1 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 2 $', True),
+            (".//li/a/span", '^Fig. 1$', True),
+            (".//li/a/span", '^Figure2.2$', True),
+            (".//li/a/span", '^Table 1$', True),
+            (".//li/a/span", '^Table:2.2$', True),
+            (".//li/a/span", '^Listing 1$', True),
+            (".//li/a/span", '^Code-2.2$', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 1.1 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 1.2 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 1.3 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 1.4 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 1.1 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 1.2 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 1.3 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 1.4 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 1.1 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 1.2 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 1.3 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 1.4 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 2.1 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 2.3 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 2.4 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 2.1 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 2.3 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 2.4 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 2.1 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 2.3 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 2.4 $', True),
+            (".//div[@class='figure']/p[@class='caption']/"
+             "span[@class='caption-number']", '^Fig. 2.2 $', True),
+            (".//table/caption/span[@class='caption-number']",
+             '^Table 2.2 $', True),
+            (".//div[@class='code-block-caption']/"
+             "span[@class='caption-number']", '^Listing 2.2 $', True),
+        ],
+    }
+
+    for fname, paths in iteritems(expects):
+        parser = NslessParser()
+        parser.entity.update(html_entities.entitydefs)
+        with (app.outdir / fname).open('rb') as fp:
+            etree = ET.parse(fp, parser)
+
+        for xpath, check, be_found in paths:
+            yield check_xpath, etree, fname, xpath, check, be_found
+
+
+@gen_with_app(buildername='html', testroot='add_enumerable_node')
+def test_enumerable_node(app, status, warning):
+    app.builder.build_all()
+
+    expects = {
+        'index.html': [
+            (".//div[@class='figure']/p[@class='caption']/span[@class='caption-number']",
+             "Fig. 1", True),
+            (".//div[@class='figure']/p[@class='caption']/span[@class='caption-number']",
+             "Fig. 2", True),
+            (".//div[@class='figure']/p[@class='caption']/span[@class='caption-number']",
+             "Fig. 3", True),
+            (".//div//span[@class='caption-number']", "No.1 ", True),
+            (".//div//span[@class='caption-number']", "No.2 ", True),
+            (".//li/a/span", 'Fig. 1', True),
+            (".//li/a/span", 'Fig. 2', True),
+            (".//li/a/span", 'Fig. 3', True),
+            (".//li/a/span", 'No.1', True),
+            (".//li/a/span", 'No.2', True),
+        ],
+    }
+
+    for fname, paths in iteritems(expects):
+        parser = NslessParser()
+        parser.entity.update(html_entities.entitydefs)
+        with (app.outdir / fname).open('rb') as fp:
+            etree = ET.parse(fp, parser)
+
+        for xpath, check, be_found in paths:
+            yield check_xpath, etree, fname, xpath, check, be_found
+
+
+@with_app(buildername='html', testroot='html_extra_path')
+def test_html_extra_path(app, status, warning):
+    app.builder.build_all()
+
+    assert (app.outdir / '.htaccess').exists()
+    assert not (app.outdir / '.htpasswd').exists()
+    assert (app.outdir / 'API.html_t').exists()
+    assert (app.outdir / 'css/style.css').exists()
+    assert (app.outdir / 'rimg.png').exists()
+    assert not (app.outdir / '_build/index.html').exists()
+    assert (app.outdir / 'background.png').exists()
