@@ -24,25 +24,49 @@ from test_build_html import ENV_WARNINGS
 
 
 LATEX_WARNINGS = ENV_WARNINGS + """\
-%(root)s/footnote.txt:60: WARNING: citation not found: missing
-%(root)s/images.txt:20: WARNING: no matching candidate for image URI u'foo.\\*'
+%(root)s/index.rst:\\d+: WARNING: unknown option: &option
+%(root)s/index.rst:\\d+: WARNING: citation not found: missing
+%(root)s/index.rst:\\d+: WARNING: no matching candidate for image URI u'foo.\\*'
+%(root)s/index.rst:\\d+: WARNING: Could not lex literal_block as "c". Highlighting skipped.
 """
 
 if PY3:
     LATEX_WARNINGS = remove_unicode_literals(LATEX_WARNINGS)
 
 
-@with_app(buildername='latex', freshenv=True)  # use freshenv to check warnings
+def run_latex(outdir):
+    """Run pdflatex, xelatex, and lualatex in the outdir"""
+    cwd = os.getcwd()
+    os.chdir(outdir)
+    try:
+        latexes = ('pdflatex', 'xelatex', 'lualatex')
+        available_latexes = len(latexes)
+        for latex in latexes:
+            try:
+                os.mkdir(latex)
+                p = Popen([latex, '--interaction=nonstopmode',
+                           '-output-directory=%s' % latex, 'SphinxTests.tex'],
+                          stdout=PIPE, stderr=PIPE)
+            except OSError:  # most likely the latex executable was not found
+                available_latexes -= 1
+            else:
+                stdout, stderr = p.communicate()
+                if p.returncode != 0:
+                    print(stdout)
+                    print(stderr)
+                    assert False, '%s exited with return code %s' % (
+                        latex, p.returncode)
+    finally:
+        os.chdir(cwd)
+
+    if available_latexes == 0:  # no latex is available, skip the test
+        raise SkipTest
+
+
+@with_app(buildername='latex')
 def test_latex(app, status, warning):
     LaTeXTranslator.ignore_missing_images = True
     app.builder.build_all()
-    latex_warnings = warning.getvalue().replace(os.sep, '/')
-    latex_warnings_exp = LATEX_WARNINGS % {
-        'root': re.escape(app.srcdir.replace(os.sep, '/'))}
-    assert re.match(latex_warnings_exp + '$', latex_warnings), \
-        'Warnings don\'t match:\n' + \
-        '--- Expected (regex):\n' + latex_warnings_exp + \
-        '--- Got:\n' + latex_warnings
 
     # file from latex_additional_files
     assert (app.outdir / 'svgimg.svg').isfile()
@@ -73,25 +97,36 @@ def test_latex(app, status, warning):
                            'seem to be installed' % filename)
 
     # now, try to run latex over it
-    cwd = os.getcwd()
-    os.chdir(app.outdir)
-    try:
-        try:
-            p = Popen(['pdflatex', '--interaction=nonstopmode',
-                       'SphinxTests.tex'], stdout=PIPE, stderr=PIPE)
-        except OSError:
-            raise SkipTest  # most likely pdflatex was not found
-        else:
-            stdout, stderr = p.communicate()
-            if p.returncode != 0:
-                print(stdout)
-                print(stderr)
-                assert False, 'latex exited with return code %s' % p.returncode
-    finally:
-        os.chdir(cwd)
+    run_latex(app.outdir)
 
 
-@with_app(buildername='latex', freshenv=True,  # use freshenv to check warnings
+@with_app(buildername='latex')
+def test_writer(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'SphinxTests.tex').text(encoding='utf8')
+
+    assert ('\\begin{sphinxfigure-in-table}\n\\centering\n\\capstart\n'
+            '\\noindent\\sphinxincludegraphics{{img}.png}\n'
+            '\\sphinxfigcaption{figure in table}\\label{markup:id7}'
+            '\\end{sphinxfigure-in-table}\\relax' in result)
+
+    assert ('\\begin{wrapfigure}{r}{0pt}\n\\centering\n'
+            '\\noindent\\sphinxincludegraphics{{rimg}.png}\n'
+            '\\caption{figure with align option}\\label{markup:id8}'
+            '\\end{wrapfigure}' in result)
+
+    assert ('\\begin{wrapfigure}{r}{0.500\\linewidth}\n\\centering\n'
+            '\\noindent\\sphinxincludegraphics{{rimg}.png}\n'
+            '\\caption{figure with align \\& figwidth option}\\label{markup:id9}'
+            '\\end{wrapfigure}' in result)
+
+    assert ('\\begin{wrapfigure}{r}{3cm}\n\\centering\n'
+            '\\noindent\\sphinxincludegraphics[width=3cm]{{rimg}.png}\n'
+            '\\caption{figure with align \\& width option}\\label{markup:id10}'
+            '\\end{wrapfigure}' in result)
+
+
+@with_app(buildername='latex',
           confoverrides={'latex_documents': [
               ('contents', 'SphinxTests.tex', 'Sphinx Tests Documentation',
                'Georg Brandl \\and someone else', 'howto'),
@@ -100,13 +135,6 @@ def test_latex(app, status, warning):
 def test_latex_howto(app, status, warning):
     LaTeXTranslator.ignore_missing_images = True
     app.builder.build_all()
-    latex_warnings = warning.getvalue().replace(os.sep, '/')
-    latex_warnings_exp = LATEX_WARNINGS % {
-        'root': re.escape(app.srcdir.replace(os.sep, '/'))}
-    assert re.match(latex_warnings_exp + '$', latex_warnings), \
-        'Warnings don\'t match:\n' + \
-        '--- Expected (regex):\n' + latex_warnings_exp + \
-        '--- Got:\n' + latex_warnings
 
     # file from latex_additional_files
     assert (app.outdir / 'svgimg.svg').isfile()
@@ -137,23 +165,20 @@ def test_latex_howto(app, status, warning):
                            'seem to be installed' % filename)
 
     # now, try to run latex over it
-    cwd = os.getcwd()
-    os.chdir(app.outdir)
-    try:
-        try:
-            p = Popen(['pdflatex', '--interaction=nonstopmode',
-                       'SphinxTests.tex'], stdout=PIPE, stderr=PIPE)
-        except OSError:
-            raise SkipTest  # most likely pdflatex was not found
-        else:
-            stdout, stderr = p.communicate()
-            if p.returncode != 0:
-                print(stdout)
-                print(stderr)
-                app.cleanup()
-                assert False, 'latex exited with return code %s' % p.returncode
-    finally:
-        os.chdir(cwd)
+    run_latex(app.outdir)
+
+
+@with_app(buildername='latex', testroot='warnings', freshenv=True)
+def test_latex_warnings(app, status, warning):
+    app.builder.build_all()
+
+    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings_exp = LATEX_WARNINGS % {
+        'root': re.escape(app.srcdir.replace(os.sep, '/'))}
+    assert re.match(warnings_exp + '$', warnings), \
+        'Warnings don\'t match:\n' + \
+        '--- Expected (regex):\n' + warnings_exp + \
+        '--- Got:\n' + warnings
 
 
 @with_app(buildername='latex', testroot='numfig',
@@ -164,10 +189,10 @@ def test_numref(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig. }}' in result
+    assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig.\\@ }}' in result
     assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Table }}' in result
     assert '\\SetupFloatingEnvironment{literal-block}{name=Listing }' in result
-    assert '\\hyperref[index:fig1]{Fig. \\ref{index:fig1}}' in result
+    assert '\\hyperref[index:fig1]{Fig.\\@ \\ref{index:fig1}}' in result
     assert '\\hyperref[baz:fig22]{Figure\\ref{baz:fig22}}' in result
     assert '\\hyperref[index:table-1]{Table \\ref{index:table-1}}' in result
     assert '\\hyperref[baz:table22]{Table:\\ref{baz:table22}}' in result
@@ -215,11 +240,11 @@ def test_numref_with_prefix2(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Figure:}}' in result
-    assert '\\def\\fnum@figure{\\figurename\\thefigure.}' in result
+    assert '\\def\\fnum@figure{\\figurename\\thefigure.\\@}' in result
     assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Tab\\_}}' in result
     assert '\\def\\fnum@table{\\tablename\\thetable:}' in result
     assert '\\SetupFloatingEnvironment{literal-block}{name=Code-}' in result
-    assert '\\hyperref[index:fig1]{Figure:\\ref{index:fig1}.}' in result
+    assert '\\hyperref[index:fig1]{Figure:\\ref{index:fig1}.\\@}' in result
     assert '\\hyperref[baz:fig22]{Figure\\ref{baz:fig22}}' in result
     assert '\\hyperref[index:table-1]{Tab\\_\\ref{index:table-1}:}' in result
     assert '\\hyperref[baz:table22]{Table:\\ref{baz:table22}}' in result
@@ -227,25 +252,6 @@ def test_numref_with_prefix2(app, status, warning):
     assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
 
 
-@with_app(buildername='latex', testroot='numfig',
-          confoverrides={'numfig': True, 'language': 'el'})
-def test_numref_with_language_el(app, status, warning):
-    app.builder.build_all()
-    result = (app.outdir / 'Python.tex').text(encoding='utf8')
-    print(result)
-    print(status.getvalue())
-    print(warning.getvalue())
-    assert '\\addto\\captionsgreek{\\renewcommand{\\figurename}{Fig. }}' in result
-    assert '\\addto\\captionsgreek{\\renewcommand{\\tablename}{Table }}' in result
-    assert '\\SetupFloatingEnvironment{literal-block}{name=Listing }' in result
-    assert '\\hyperref[index:fig1]{Fig. \\ref{index:fig1}}' in result
-    assert '\\hyperref[baz:fig22]{Figure\\ref{baz:fig22}}' in result
-    assert '\\hyperref[index:table-1]{Table \\ref{index:table-1}}' in result
-    assert '\\hyperref[baz:table22]{Table:\\ref{baz:table22}}' in result
-    assert '\\hyperref[index:code-1]{Listing \\ref{index:code-1}}' in result
-    assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
-
-
 @with_app(buildername='latex', testroot='numfig',
           confoverrides={'numfig': True, 'language': 'ja'})
 def test_numref_with_language_ja(app, status, warning):
@@ -265,25 +271,6 @@ def test_numref_with_language_ja(app, status, warning):
     assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
 
 
-@with_app(buildername='latex', testroot='numfig',
-          confoverrides={'numfig': True, 'language': 'ru', 'latex_elements': {'babel': ''}})
-def test_numref_on_bable_disabled(app, status, warning):
-    app.builder.build_all()
-    result = (app.outdir / 'Python.tex').text(encoding='utf8')
-    print(result)
-    print(status.getvalue())
-    print(warning.getvalue())
-    assert '\\renewcommand{\\figurename}{Fig. }' in result
-    assert '\\renewcommand{\\tablename}{Table }' in result
-    assert '\\SetupFloatingEnvironment{literal-block}{name=Listing }' in result
-    assert '\\hyperref[index:fig1]{Fig. \\ref{index:fig1}}' in result
-    assert '\\hyperref[baz:fig22]{Figure\\ref{baz:fig22}}' in result
-    assert '\\hyperref[index:table-1]{Table \\ref{index:table-1}}' in result
-    assert '\\hyperref[baz:table22]{Table:\\ref{baz:table22}}' in result
-    assert '\\hyperref[index:code-1]{Listing \\ref{index:code-1}}' in result
-    assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
-
-
 @with_app(buildername='latex')
 def test_latex_add_latex_package(app, status, warning):
     app.add_latex_package('foo')
@@ -294,26 +281,124 @@ def test_latex_add_latex_package(app, status, warning):
     assert '\\usepackage[baz]{bar}' in result
 
 
-@with_app(buildername='latex', testroot='contentsname')
-def test_contentsname(app, status, warning):
+@with_app(buildername='latex', testroot='latex-babel')
+def test_babel_with_no_language_settings(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'Python.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert ('\\addto\\captionsenglish{\\renewcommand{\\contentsname}{Table of content}}'
+    assert '\\documentclass[letterpaper,10pt,english]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Bjarne]{fncychap}' in result
+    assert ('\\addto\\captionsenglish{\\renewcommand{\\contentsname}{Table of content}}\n'
             in result)
+    assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig.\\@ }}\n' in result
+    assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Table.\\@ }}\n' in result
+    assert '\\addto\\extrasenglish{\\def\\pageautorefname{page}}\n' in result
+    assert '\\shorthandoff' not in result
 
 
-@with_app(buildername='latex', testroot='contentsname',
-          confoverrides={'language': 'ja'})
-def test_contentsname_with_language_ja(app, status, warning):
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'de'})
+def test_babel_with_language_de(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'Python.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\renewcommand{\\contentsname}{Table of content}' in result
+    assert '\\documentclass[letterpaper,10pt,ngerman]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsngerman{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsngerman{\\renewcommand{\\figurename}{Fig.\\@ }}\n' in result
+    assert '\\addto\\captionsngerman{\\renewcommand{\\tablename}{Table.\\@ }}\n' in result
+    assert '\\addto\\extrasngerman{\\def\\pageautorefname{page}}\n' in result
+    assert '\\shorthandoff{"}' in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'ru'})
+def test_babel_with_language_ru(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,russian]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' not in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsrussian{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsrussian{\\renewcommand{\\figurename}{Fig.\\@ }}\n' in result
+    assert '\\addto\\captionsrussian{\\renewcommand{\\tablename}{Table.\\@ }}\n' in result
+    assert '\\addto\\extrasrussian{\\def\\pageautorefname{page}}\n' in result
+    assert '\\shorthandoff' not in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'tr'})
+def test_babel_with_language_tr(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,turkish]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsturkish{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsturkish{\\renewcommand{\\figurename}{Fig.\\@ }}\n' in result
+    assert '\\addto\\captionsturkish{\\renewcommand{\\tablename}{Table.\\@ }}\n' in result
+    assert '\\addto\\extrasturkish{\\def\\pageautorefname{sayfa}}\n' in result
+    assert '\\shorthandoff{=}' in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'ja'})
+def test_babel_with_language_ja(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,dvipdfmx]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' not in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' not in result
+    assert '\\renewcommand{\\contentsname}{Table of content}\n' in result
+    assert '\\renewcommand{\\figurename}{Fig.\\@ }\n' in result
+    assert '\\renewcommand{\\tablename}{Table.\\@ }\n' in result
+    assert u'\\def\\pageautorefname{ページ}\n' in result
+    assert '\\shorthandoff' not in result
+
+
+@with_app(buildername='latex', testroot='latex-babel',
+          confoverrides={'language': 'unknown'})
+def test_babel_with_unknown_language(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\documentclass[letterpaper,10pt,english]{sphinxmanual}' in result
+    assert '\\usepackage{babel}' in result
+    assert '\\usepackage{times}' in result
+    assert '\\usepackage[Sonny]{fncychap}' in result
+    assert ('\\addto\\captionsenglish{\\renewcommand{\\contentsname}{Table of content}}\n'
+            in result)
+    assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig.\\@ }}\n' in result
+    assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Table.\\@ }}\n' in result
+    assert '\\addto\\extrasenglish{\\def\\pageautorefname{page}}\n' in result
+    assert '\\shorthandoff' not in result
+
+    assert "WARNING: no Babel option known for language 'unknown'" in warning.getvalue()
 
 
 @with_app(buildername='latex')
@@ -323,18 +408,18 @@ def test_footnote(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\footnote[1]{\nnumbered\n}' in result
-    assert '\\footnote[2]{\nauto numbered\n}' in result
-    assert '\\footnote[3]{\nnamed\n}' in result
-    assert '{\\hyperref[footnote:bar]{\\emph{{[}bar{]}}}}' in result
+    assert '\\footnote[1]{\sphinxAtStartFootnote%\nnumbered\n}' in result
+    assert '\\footnote[2]{\sphinxAtStartFootnote%\nauto numbered\n}' in result
+    assert '\\footnote[3]{\sphinxAtStartFootnote%\nnamed\n}' in result
+    assert '{\\hyperref[footnote:bar]{\\sphinxcrossref{{[}bar{]}}}}' in result
     assert '\\bibitem[bar]{bar}{\\phantomsection\\label{footnote:bar} ' in result
     assert '\\bibitem[bar]{bar}{\\phantomsection\\label{footnote:bar} \ncite' in result
     assert '\\bibitem[bar]{bar}{\\phantomsection\\label{footnote:bar} \ncite\n}' in result
     assert '\\capstart\\caption{Table caption \\protect\\footnotemark[4]}' in result
     assert 'name \\protect\\footnotemark[5]' in result
     assert ('\\end{threeparttable}\n\n'
-            '\\footnotetext[4]{\nfootnotes in table caption\n}'
-            '\\footnotetext[5]{\nfootnotes in table\n}' in result)
+            '\\footnotetext[4]{\sphinxAtStartFootnote%\nfootnotes in table caption\n}'
+            '\\footnotetext[5]{\sphinxAtStartFootnote%\nfootnotes in table\n}' in result)
 
 
 @with_app(buildername='latex', testroot='footnotes')
@@ -345,21 +430,24 @@ def test_reference_in_caption(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert ('\\caption{This is the figure caption with a reference to \\label{index:id2}'
-            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}.}' in result)
+            '{\\hyperref[index:authoryear]{\\sphinxcrossref{{[}AuthorYear{]}}}}.}' in result)
     assert '\\chapter{The section with a reference to {[}AuthorYear{]}}' in result
     assert '\\caption{The table title with a reference to {[}AuthorYear{]}}' in result
     assert '\\paragraph{The rubric title with a reference to {[}AuthorYear{]}}' in result
     assert ('\\chapter{The section with a reference to \\protect\\footnotemark[4]}\n'
             '\\label{index:the-section-with-a-reference-to}'
-            '\\footnotetext[4]{\nFootnote in section\n}' in result)
+            '\\footnotetext[4]{\sphinxAtStartFootnote%\nFootnote in section\n}' in result)
     assert ('\\caption{This is the figure caption with a footnote to '
-            '\\protect\\footnotemark[6].}\end{figure}\n'
-            '\\footnotetext[6]{\nFootnote in caption\n}')in result
+            '\\protect\\footnotemark[6].}\label{index:id23}\end{figure}\n'
+            '\\footnotetext[6]{\sphinxAtStartFootnote%\nFootnote in caption\n}')in result
     assert ('\\caption{footnote \\protect\\footnotemark[7] '
             'in caption of normal table}') in result
-    assert '\\end{threeparttable}\n\n\\footnotetext[7]{\nFoot note in table\n}' in result
-    assert '\\caption{footnote \\protect\\footnotemark[8] in caption of longtable}' in result
-    assert '\end{longtable}\n\n\\footnotetext[8]{\nFoot note in longtable\n}' in result
+    assert ('\\end{threeparttable}\n\n\\footnotetext[7]{\sphinxAtStartFootnote%\n'
+            'Foot note in table\n}' in result)
+    assert ('\\caption{footnote \\protect\\footnotemark[8] in caption of longtable}'
+            in result)
+    assert ('\end{longtable}\n\n\\footnotetext[8]{\sphinxAtStartFootnote%\n'
+            'Foot note in longtable\n}' in result)
 
 
 @with_app(buildername='latex', testroot='footnotes',
@@ -370,29 +458,32 @@ def test_latex_show_urls_is_inline(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'Same footnote number \\footnote[1]{\nfootnote in bar\n} in bar.rst' in result
-    assert 'Auto footnote number \\footnote[1]{\nfootnote in baz\n} in baz.rst' in result
+    assert ('Same footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in bar\n} in bar.rst' in result)
+    assert ('Auto footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in baz\n} in baz.rst' in result)
     assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
             '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
-            '{\\emph{The section with a reference to \\phantomsection\\label{index:id1}'
-            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}}}}' in result)
+            '{\\sphinxcrossref{The section with a reference to '
+            '\\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\sphinxcrossref{{[}AuthorYear{]}}}}}}}' in result)
     assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
-            '\\string-with\\string-a\\string-reference\\string-to]{\\emph{The section '
-            'with a reference to }}}' in result)
-    assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
-    assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
+            '\\string-with\\string-a\\string-reference\\string-to]'
+            '{\\sphinxcrossref{The section with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[2]{\sphinxAtStartFootnote%\nFirst\n}' in result
+    assert 'Second footnote: \\footnote[1]{\sphinxAtStartFootnote%\nSecond\n}' in result
     assert '\\href{http://sphinx-doc.org/}{Sphinx} (http://sphinx-doc.org/)' in result
-    assert 'Third footnote: \\footnote[3]{\nThird\n}' in result
+    assert 'Third footnote: \\footnote[3]{\sphinxAtStartFootnote%\nThird\n}' in result
     assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde} '
             '(http://sphinx-doc.org/\\textasciitilde{}test/)' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term} (http://sphinx-doc.org/)}] '
             '\\leavevmode\nDescription' in result)
     assert ('\\item[{Footnote in term \\protect\\footnotemark[5]}] '
-            '\\leavevmode\\footnotetext[5]{\nFootnote in term\n}\nDescription' in result)
+            '\\leavevmode\\footnotetext[5]{\sphinxAtStartFootnote%\n'
+            'Footnote in term\n}\nDescription' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist} '
             '(http://sphinx-doc.org/)}] \\leavevmode\nDescription' in result)
-    assert ('\\href{https://github.com/sphinx-doc/sphinx}'
-            '{https://github.com/sphinx-doc/sphinx}\n' in result)
+    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}' in result)
 
@@ -405,31 +496,38 @@ def test_latex_show_urls_is_footnote(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'Same footnote number \\footnote[1]{\nfootnote in bar\n} in bar.rst' in result
-    assert 'Auto footnote number \\footnote[2]{\nfootnote in baz\n} in baz.rst' in result
+    assert ('Same footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in bar\n} in bar.rst' in result)
+    assert ('Auto footnote number \\footnote[2]{\sphinxAtStartFootnote%\n'
+            'footnote in baz\n} in baz.rst' in result)
     assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
             '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
-            '{\\emph{The section with a reference to \\phantomsection\\label{index:id1}'
-            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}}}}' in result)
+            '{\\sphinxcrossref{The section with a reference '
+            'to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\sphinxcrossref{{[}AuthorYear{]}}}}}}}' in result)
     assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
-            '\\string-with\\string-a\\string-reference\\string-to]{\\emph{The section '
-            'with a reference to }}}' in result)
-    assert 'First footnote: \\footnote[3]{\nFirst\n}' in result
-    assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
+            '\\string-with\\string-a\\string-reference\\string-to]'
+            '{\\sphinxcrossref{The section with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[3]{\sphinxAtStartFootnote%\nFirst\n}' in result
+    assert 'Second footnote: \\footnote[1]{\sphinxAtStartFootnote%\nSecond\n}' in result
     assert ('\\href{http://sphinx-doc.org/}{Sphinx}'
-            '\\footnote[4]{\nhttp://sphinx-doc.org/\n}' in result)
-    assert 'Third footnote: \\footnote[6]{\nThird\n}' in result
+            '\\footnote[4]{\sphinxAtStartFootnote%\n'
+            '\\nolinkurl{http://sphinx-doc.org/}\n}' in result)
+    assert 'Third footnote: \\footnote[6]{\sphinxAtStartFootnote%\nThird\n}' in result
     assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde}'
-            '\\footnote[5]{\nhttp://sphinx-doc.org/\\textasciitilde{}test/\n}' in result)
+            '\\footnote[5]{\sphinxAtStartFootnote%\n'
+            '\\nolinkurl{http://sphinx-doc.org/~test/}\n}' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}\\protect\\footnotemark[8]}] '
-            '\\leavevmode\\footnotetext[8]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
+            '\\leavevmode\\footnotetext[8]{\sphinxAtStartFootnote%\n'
+            '\\nolinkurl{http://sphinx-doc.org/}\n}\nDescription' in result)
     assert ('\\item[{Footnote in term \\protect\\footnotemark[10]}] '
-            '\\leavevmode\\footnotetext[10]{\nFootnote in term\n}\nDescription' in result)
+            '\\leavevmode\\footnotetext[10]{\sphinxAtStartFootnote%\n'
+            'Footnote in term\n}\nDescription' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}\\protect'
             '\\footnotemark[9]}] '
-            '\\leavevmode\\footnotetext[9]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
-    assert ('\\href{https://github.com/sphinx-doc/sphinx}'
-            '{https://github.com/sphinx-doc/sphinx}\n' in result)
+            '\\leavevmode\\footnotetext[9]{\sphinxAtStartFootnote%\n'
+            '\\nolinkurl{http://sphinx-doc.org/}\n}\nDescription' in result)
+    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}\n' in result)
 
@@ -442,28 +540,31 @@ def test_latex_show_urls_is_no(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'Same footnote number \\footnote[1]{\nfootnote in bar\n} in bar.rst' in result
-    assert 'Auto footnote number \\footnote[1]{\nfootnote in baz\n} in baz.rst' in result
+    assert ('Same footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in bar\n} in bar.rst' in result)
+    assert ('Auto footnote number \\footnote[1]{\sphinxAtStartFootnote%\n'
+            'footnote in baz\n} in baz.rst' in result)
     assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
             '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
-            '{\\emph{The section with a reference to \\phantomsection\\label{index:id1}'
-            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}}}}' in result)
+            '{\\sphinxcrossref{The section with a reference '
+            'to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\sphinxcrossref{{[}AuthorYear{]}}}}}}}' in result)
     assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
-            '\\string-with\\string-a\\string-reference\\string-to]{\\emph{The section '
-            'with a reference to }}}' in result)
-    assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
-    assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
+            '\\string-with\\string-a\\string-reference\\string-to]'
+            '{\\sphinxcrossref{The section with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[2]{\sphinxAtStartFootnote%\nFirst\n}' in result
+    assert 'Second footnote: \\footnote[1]{\sphinxAtStartFootnote%\nSecond\n}' in result
     assert '\\href{http://sphinx-doc.org/}{Sphinx}' in result
-    assert 'Third footnote: \\footnote[3]{\nThird\n}' in result
+    assert 'Third footnote: \\footnote[3]{\sphinxAtStartFootnote%\nThird\n}' in result
     assert '\\href{http://sphinx-doc.org/~test/}{URL including tilde}' in result
     assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}}] '
             '\\leavevmode\nDescription' in result)
     assert ('\\item[{Footnote in term \\protect\\footnotemark[5]}] '
-            '\\leavevmode\\footnotetext[5]{\nFootnote in term\n}\nDescription' in result)
+            '\\leavevmode\\footnotetext[5]{\sphinxAtStartFootnote%\n'
+            'Footnote in term\n}\nDescription' in result)
     assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}}] '
             '\\leavevmode\nDescription' in result)
-    assert ('\\href{https://github.com/sphinx-doc/sphinx}'
-            '{https://github.com/sphinx-doc/sphinx}\n' in result)
+    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}\n' in result)
 
@@ -475,12 +576,13 @@ def test_image_in_section(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert ('\chapter[Test section]'
-            '{\includegraphics[width=15pt,height=15pt]{{pic}.png} Test section}'
+    assert ('\\chapter[Test section]{\\lowercase{\\sphinxincludegraphics'
+            '[width=15pt,height=15pt]}{{pic}.png} Test section}'
             in result)
-    assert ('\chapter[Other {[}blah{]} section]{Other {[}blah{]} '
-            '\includegraphics[width=15pt,height=15pt]{{pic}.png} section}' in result)
-    assert ('\chapter{Another section}' in result)
+    assert ('\\chapter[Other {[}blah{]} section]{Other {[}blah{]} '
+            '\\lowercase{\\sphinxincludegraphics[width=15pt,height=15pt]}'
+            '{{pic}.png} section}' in result)
+    assert ('\\chapter{Another section}' in result)
 
 
 @with_app(buildername='latex', confoverrides={'latex_logo': 'notfound.jpg'})
@@ -540,3 +642,47 @@ def test_toctree_without_maxdepth(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert '\\setcounter{tocdepth}' not in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': None})
+def test_latex_toplevel_sectioning_is_None(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\chapter{Foo}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': 'part'})
+def test_latex_toplevel_sectioning_is_part(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\part{Foo}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': 'chapter'})
+def test_latex_toplevel_sectioning_is_chapter(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\chapter{Foo}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'latex_toplevel_sectioning': 'section'})
+def test_latex_toplevel_sectioning_is_section(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\section{Foo}' in result
