@@ -13,6 +13,8 @@ from __future__ import print_function
 
 import os
 import re
+import pickle
+from docutils import nodes
 from subprocess import Popen, PIPE
 from xml.etree import ElementTree
 
@@ -20,9 +22,9 @@ from babel.messages import pofile
 from nose.tools import assert_equal
 from six import string_types
 
-from util import tempdir, rootdir, path, gen_with_app, SkipTest, \
+from util import tempdir, rootdir, path, gen_with_app, with_app, SkipTest, \
     assert_re_search, assert_not_re_search, assert_in, assert_not_in, \
-    assert_startswith
+    assert_startswith, assert_node, repr_as
 
 
 root = tempdir / 'test-intl'
@@ -118,7 +120,7 @@ def test_text_builder(app, status, warning):
 
     # --- warnings in translation
 
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     warning_expr = u'.*/warnings.txt:4: ' \
                    u'WARNING: Inline literal start-string without end-string.\n'
     yield assert_re_search, warning_expr, warnings
@@ -155,7 +157,7 @@ def test_text_builder(app, status, warning):
               u"\n[100] THIS IS A NUMBERED FOOTNOTE.\n")
     yield assert_equal, result, expect
 
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     warning_fmt = u'.*/refs_inconsistency.txt:\\d+: ' \
                   u'WARNING: inconsistent %s in translated message\n'
     expected_warning_expr = (
@@ -176,7 +178,7 @@ def test_text_builder(app, status, warning):
               u"\n<SYSTEM MESSAGE:")
     yield assert_startswith, result, expect
 
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     expected_warning_expr = u'.*/literalblock.txt:\\d+: ' \
                             u'WARNING: Literal block expected; none found.'
     yield assert_re_search, expected_warning_expr, warnings
@@ -208,7 +210,7 @@ def test_text_builder(app, status, warning):
               u"\n   THE CORRESPONDING GLOSSARY #2\n"
               u"\nLINK TO *SOME NEW TERM*.\n")
     yield assert_equal, result, expect
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     yield assert_not_in, 'term not in glossary', warnings
 
     # --- glossary term inconsistencies: regression test for #1090
@@ -219,7 +221,7 @@ def test_text_builder(app, status, warning):
               u"\n1. LINK TO *SOME NEW TERM*.\n")
     yield assert_equal, result, expect
 
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     expected_warning_expr = (
         u'.*/glossary_terms_inconsistency.txt:\\d+: '
         u'WARNING: inconsistent term references in translated message\n')
@@ -485,7 +487,7 @@ def test_xml_builder(app, status, warning):
            None,
            ['ref'])
 
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     warning_expr = u'.*/footnote.xml:\\d*: SEVERE: Duplicate ID: ".*".\n'
     yield assert_not_re_search, warning_expr, warnings
 
@@ -616,7 +618,7 @@ def test_xml_builder(app, status, warning):
            ['same-type-links', 'i18n-role-xref'])
 
     # warnings
-    warnings = warning.getvalue().replace(os.sep, '/')
+    warnings = getwarning(warning)
     yield assert_not_in, 'term not in glossary', warnings
     yield assert_not_in, 'undefined label', warnings
     yield assert_not_in, 'unknown document', warnings
@@ -678,7 +680,7 @@ def test_xml_builder(app, status, warning):
 def test_additional_targets_should_not_be_translated(app, status, warning):
     app.builder.build_all()
 
-    ## literalblock.txt
+    # [literalblock.txt]
     result = (app.outdir / 'literalblock.html').text(encoding='utf-8')
 
     # title should be translated
@@ -705,7 +707,7 @@ def test_additional_targets_should_not_be_translated(app, status, warning):
         """<span class="c1"># sys importing</span>""")
     yield assert_count(expected_expr, result, 1)
 
-    ## raw.txt
+    # [raw.txt]
 
     result = (app.outdir / 'raw.html').text(encoding='utf-8')
 
@@ -713,7 +715,7 @@ def test_additional_targets_should_not_be_translated(app, status, warning):
     expected_expr = """<iframe src="http://sphinx-doc.org"></iframe></div>"""
     yield assert_count(expected_expr, result, 1)
 
-    ## figure.txt
+    # [figure.txt]
 
     result = (app.outdir / 'figure.html').text(encoding='utf-8')
 
@@ -739,7 +741,7 @@ def test_additional_targets_should_not_be_translated(app, status, warning):
 def test_additional_targets_should_be_translated(app, status, warning):
     app.builder.build_all()
 
-    ## literalblock.txt
+    # [literalblock.txt]
     result = (app.outdir / 'literalblock.html').text(encoding='utf-8')
 
     # title should be translated
@@ -766,7 +768,7 @@ def test_additional_targets_should_be_translated(app, status, warning):
         """<span class="c1"># SYS IMPORTING</span>""")
     yield assert_count(expected_expr, result, 1)
 
-    ## raw.txt
+    # [raw.txt]
 
     result = (app.outdir / 'raw.html').text(encoding='utf-8')
 
@@ -774,7 +776,7 @@ def test_additional_targets_should_be_translated(app, status, warning):
     expected_expr = """<iframe src="HTTP://SPHINX-DOC.ORG"></iframe></div>"""
     yield assert_count(expected_expr, result, 1)
 
-    ## figure.txt
+    # [figure.txt]
 
     result = (app.outdir / 'figure.html').text(encoding='utf-8')
 
@@ -794,3 +796,91 @@ def test_references(app, status, warning):
     warnings = warning.getvalue().replace(os.sep, '/')
     warning_expr = u'refs.txt:\\d+: ERROR: Unknown target name:'
     yield assert_count(warning_expr, warnings, 0)
+
+
+@with_app(buildername='dummy', testroot='image-glob', confoverrides={'language': 'xx'})
+def test_image_glob_intl(app, status, warning):
+    app.builder.build_all()
+
+    # index.rst
+    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
+
+    assert_node(doctree[0][1], nodes.image, uri='rimg.xx.png',
+                candidates={'*': 'rimg.xx.png'})
+
+    assert isinstance(doctree[0][2], nodes.figure)
+    assert_node(doctree[0][2][0], nodes.image, uri='rimg.xx.png',
+                candidates={'*': 'rimg.xx.png'})
+
+    assert_node(doctree[0][3], nodes.image, uri='img.*',
+                candidates={'application/pdf': 'img.pdf',
+                            'image/gif': 'img.gif',
+                            'image/png': 'img.png'})
+
+    assert isinstance(doctree[0][4], nodes.figure)
+    assert_node(doctree[0][4][0], nodes.image, uri='img.*',
+                candidates={'application/pdf': 'img.pdf',
+                            'image/gif': 'img.gif',
+                            'image/png': 'img.png'})
+
+    # subdir/index.rst
+    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
+
+    assert_node(doctree[0][1], nodes.image, uri='subdir/rimg.xx.png',
+                candidates={'*': 'subdir/rimg.xx.png'})
+
+    assert_node(doctree[0][2], nodes.image, uri='subdir/svgimg.*',
+                candidates={'application/pdf': 'subdir/svgimg.pdf',
+                            'image/svg+xml': 'subdir/svgimg.xx.svg'})
+
+    assert isinstance(doctree[0][3], nodes.figure)
+    assert_node(doctree[0][3][0], nodes.image, uri='subdir/svgimg.*',
+                candidates={'application/pdf': 'subdir/svgimg.pdf',
+                            'image/svg+xml': 'subdir/svgimg.xx.svg'})
+
+
+@with_app(buildername='dummy', testroot='image-glob',
+          confoverrides={'language': 'xx',
+                         'figure_language_filename': u'{root}{ext}.{language}'})
+def test_image_glob_intl_using_figure_language_filename(app, status, warning):
+    app.builder.build_all()
+
+    # index.rst
+    doctree = pickle.loads((app.doctreedir / 'index.doctree').bytes())
+
+    assert_node(doctree[0][1], nodes.image, uri='rimg.png.xx',
+                candidates={'*': 'rimg.png.xx'})
+
+    assert isinstance(doctree[0][2], nodes.figure)
+    assert_node(doctree[0][2][0], nodes.image, uri='rimg.png.xx',
+                candidates={'*': 'rimg.png.xx'})
+
+    assert_node(doctree[0][3], nodes.image, uri='img.*',
+                candidates={'application/pdf': 'img.pdf',
+                            'image/gif': 'img.gif',
+                            'image/png': 'img.png'})
+
+    assert isinstance(doctree[0][4], nodes.figure)
+    assert_node(doctree[0][4][0], nodes.image, uri='img.*',
+                candidates={'application/pdf': 'img.pdf',
+                            'image/gif': 'img.gif',
+                            'image/png': 'img.png'})
+
+    # subdir/index.rst
+    doctree = pickle.loads((app.doctreedir / 'subdir/index.doctree').bytes())
+
+    assert_node(doctree[0][1], nodes.image, uri='subdir/rimg.png',
+                candidates={'*': 'subdir/rimg.png'})
+
+    assert_node(doctree[0][2], nodes.image, uri='subdir/svgimg.*',
+                candidates={'application/pdf': 'subdir/svgimg.pdf',
+                            'image/svg+xml': 'subdir/svgimg.svg'})
+
+    assert isinstance(doctree[0][3], nodes.figure)
+    assert_node(doctree[0][3][0], nodes.image, uri='subdir/svgimg.*',
+                candidates={'application/pdf': 'subdir/svgimg.pdf',
+                            'image/svg+xml': 'subdir/svgimg.svg'})
+
+
+def getwarning(warnings):
+    return repr_as(warnings.getvalue().replace(os.sep, '/'), '<warnings>')
