@@ -10,6 +10,7 @@
 """
 
 import re
+import pickle
 
 from docutils import frontend, utils, nodes
 from docutils.parsers import rst
@@ -18,7 +19,7 @@ from sphinx.util import texescape
 from sphinx.writers.html import HTMLWriter, SmartyPantsHTMLTranslator
 from sphinx.writers.latex import LaTeXWriter, LaTeXTranslator
 
-from util import TestApp
+from util import TestApp, with_app, assert_node
 
 
 app = settings = parser = None
@@ -40,6 +41,7 @@ def setup_module():
 def teardown_module():
     app.cleanup()
 
+
 # since we're not resolving the markup afterwards, these nodes may remain
 class ForgivingTranslator:
     def visit_pending_xref(self, node):
@@ -73,11 +75,12 @@ def verify_re(rst, html_expected, latex_expected):
 
     if latex_expected:
         latex_translator = ForgivingLaTeXTranslator(document, app.builder)
-        latex_translator.first_document = -1 # don't write \begin{document}
+        latex_translator.first_document = -1  # don't write \begin{document}
         document.walkabout(latex_translator)
         latex_translated = ''.join(latex_translator.body).strip()
         assert re.match(latex_expected, latex_translated), 'from ' + repr(rst)
 
+
 def verify(rst, html_expected, latex_expected):
     if html_expected:
         html_expected = re.escape(html_expected) + '$'
@@ -89,33 +92,33 @@ def verify(rst, html_expected, latex_expected):
 def test_inline():
     # correct interpretation of code with whitespace
     _html = ('<p><code class="(samp )?docutils literal"><span class="pre">'
-             'code</span>&nbsp;&nbsp; <span class="pre">sample</span></code></p>')
-    yield verify_re, '``code   sample``', _html, r'\\code{code   sample}'
-    yield verify_re, ':samp:`code   sample`', _html, r'\\code{code   sample}'
+             'code</span>&#160;&#160; <span class="pre">sample</span></code></p>')
+    yield verify_re, '``code   sample``', _html, r'\\sphinxcode{code   sample}'
+    yield verify_re, ':samp:`code   sample`', _html, r'\\sphinxcode{code   sample}'
 
     # interpolation of braces in samp and file roles (HTML only)
     yield (verify, ':samp:`a{b}c`',
            '<p><code class="samp docutils literal"><span class="pre">a</span>'
            '<em><span class="pre">b</span></em>'
            '<span class="pre">c</span></code></p>',
-           '\\code{a\\emph{b}c}')
+           '\\sphinxcode{a\\emph{b}c}')
 
     # interpolation of arrows in menuselection
     yield (verify, ':menuselection:`a --> b`',
            u'<p><span class="menuselection">a \N{TRIANGULAR BULLET} b</span></p>',
-           '\\emph{a \\(\\rightarrow\\) b}')
+           '\\sphinxmenuselection{a \\(\\rightarrow\\) b}')
 
     # interpolation of ampersands in guilabel/menuselection
     yield (verify, ':guilabel:`&Foo -&&- &Bar`',
            u'<p><span class="guilabel"><span class="accelerator">F</span>oo '
            '-&amp;- <span class="accelerator">B</span>ar</span></p>',
-           r'\emph{\underline{F}oo -\&- \underline{B}ar}')
+           r'\sphinxmenuselection{\sphinxaccelerator{F}oo -\&- \sphinxaccelerator{B}ar}')
 
     # non-interpolation of dashes in option role
     yield (verify_re, ':option:`--with-option`',
            '<p><code( class="xref std std-option docutils literal")?>'
            '<span class="pre">--with-option</span></code></p>$',
-           r'\\code{-{-}with-option}$')
+           r'\\sphinxcode{-{-}with-option}$')
 
     # verify smarty-pants quotes
     yield verify, '"John"', '<p>&#8220;John&#8221;</p>', "``John''"
@@ -123,13 +126,14 @@ def test_inline():
     yield (verify, '``"John"``',
            '<p><code class="docutils literal"><span class="pre">'
            '&quot;John&quot;</span></code></p>',
-           '\\code{"John"}')
+           '\\sphinxcode{"John"}')
 
     # verify classes for inline roles
     yield (verify, ':manpage:`mp(1)`',
            '<p><em class="manpage">mp(1)</em></p>',
            '\\emph{\\texttt{mp(1)}}')
 
+
 def test_latex_escaping():
     # correct escaping in normal mode
     yield (verify, u'Γ\\\\∞$', None,
@@ -142,3 +146,27 @@ def test_latex_escaping():
     # in URIs
     yield (verify_re, u'`test <http://example.com/~me/>`_', None,
            r'\\href{http://example.com/~me/}{test}.*')
+
+
+@with_app(buildername='dummy', testroot='prolog')
+def test_rst_prolog(app, status, warning):
+    app.builder.build_all()
+    rst = pickle.loads((app.doctreedir / 'restructuredtext.doctree').bytes())
+    md = pickle.loads((app.doctreedir / 'markdown.doctree').bytes())
+
+    # rst_prolog
+    assert_node(rst[0], nodes.paragraph)
+    assert_node(rst[0][0], nodes.emphasis)
+    assert_node(rst[0][0][0], nodes.Text)
+    assert rst[0][0][0] == 'Hello world'
+
+    # rst_epilog
+    assert_node(rst[-1], nodes.section)
+    assert_node(rst[-1][-1], nodes.paragraph)
+    assert_node(rst[-1][-1][0], nodes.emphasis)
+    assert_node(rst[-1][-1][0][0], nodes.Text)
+    assert rst[-1][-1][0][0] == 'Good-bye world'
+
+    # rst_prolog & rst_epilog on exlucding reST parser
+    assert not md.rawsource.startswith('*Hello world*.')
+    assert not md.rawsource.endswith('*Good-bye world*.\n')
